
package de.laliluna.example;

import javax.naming.InitialContext;
import javax.naming.NamingException;


import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.cfg.Environment;

/**
 * @author hennebrueder This class garanties that only one single SessionFactory
 *         is instanciated and that the configuration is done thread safe as
 *         singleton. Actually it only wraps the Hibernate SessionFactory. When
 *         a JNDI name is configured the session is bound to to JNDI, else it is
 *         only saved locally. You are free to use any kind of JTA or Thread
 *         transactionFactories.
 */
public class InitSessionFactory {

	private static org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(InitSessionFactory.class);

	/** The single instance of hibernate SessionFactory */
	private static org.hibernate.SessionFactory sessionFactory;

	/**
     * Default constructor. It is private to guaranty singleton
     */
	private InitSessionFactory() {
	}

	static {
		final AnnotationConfiguration cfg = new AnnotationConfiguration();
		/*
         * configure Hibernate from hibernate.cfg.xml. This is expected to be in
         * the class path = "src folder" = "root folder of classes"
         */
		cfg.configure("/hibernate.cfg.xml");
		String sessionFactoryJndiName = cfg
				.getProperty(Environment.SESSION_FACTORY_NAME);
		sessionFactory = cfg.buildSessionFactory();
	}

	/**
     * Returns the single instance of the session factory
     * 
     * @return
     */
	public static SessionFactory getInstance() {
		return sessionFactory;
	}

}
