package de.laliluna.sandcastle.web;

import java.io.IOException;
import java.util.List;
import java.util.Random;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.EJBContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import de.laliluna.dialogue.SandcastleWizard;
import de.laliluna.sandcastle.Sandcastle;
import de.laliluna.sandcastle.SandcastleService;

@WebServlet(urlPatterns = "/sandcastle")
public class SandcastleServlet extends HttpServlet {

    @EJB
    SandcastleService serviceLocal;
    /* constant for wizard in session */
    public static final String WIZARD = "wizard";

    /**
     * Constructor of the object.
     */
    public SandcastleServlet() {
        super();
    }

    /**
     * Destruction of the servlet. <br>
     */
    public void destroy() {
        super.destroy();
    }

    /**
     * depending on a param creates, deletes or just lists castles
     * A Wizard is supported as well.
     *
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    private void process(HttpServletRequest request,
                         HttpServletResponse response) throws ServletException, IOException {

        String param = request.getParameter("task");
        if (param == null)
            param = "";
        if (param.equals("create"))
            serviceLocal.create(new de.laliluna.sandcastle.Sandcastle(
                    "white beach castle " + new Random().nextInt(100)));
        else if (param.equals("delete")) {
            /* find all and delete one of them */
            List<Sandcastle> castles = serviceLocal.findAll();
            if (castles.size() > 0)
                serviceLocal.delete(castles.get(0));
        } else if (param.equals("step1")) {
            SandcastleWizard wizardLocal = getSandcastleWizard();
            wizardLocal.step1();
            request.getSession().setAttribute(WIZARD, wizardLocal);
			/* let JSP render step2 */
            request.setAttribute("step", "2");
        } else if (param.equals("step2")) {
            SandcastleWizard wizardLocal = (SandcastleWizard) request.getSession().getAttribute(WIZARD);
            if (wizardLocal == null)
                throw new ServletException("you missed step 1 of the wizard");
            String name = "Caste of the wizard " + new Random().nextInt(500);
            wizardLocal.step2(name);
			/* let JSP render step3 */
            request.setAttribute("step", "3");

        } else if (param.equals("step3")) {
            SandcastleWizard wizardLocal = (SandcastleWizard) request.getSession().getAttribute(WIZARD);
            if (wizardLocal == null)
                throw new ServletException("you missed step 1 of the wizard");
            wizardLocal.step3("blue");
            request.getSession().removeAttribute(WIZARD);
        }

        List<Sandcastle> castles = serviceLocal.findAll();
        request.setAttribute("castles", castles);
        RequestDispatcher dispatcher = request
                .getRequestDispatcher("/list.jsp");
        dispatcher.forward(request, response);

    }

    private SandcastleWizard getSandcastleWizard() {
        SandcastleWizard wizardLocal = null;
        try {
            wizardLocal = (SandcastleWizard) new InitialContext().lookup("java:module/SandcastleWizardBean");
        } catch (NamingException e) {
            e.printStackTrace();
        }
        return wizardLocal;
    }

    /**
     * The doGet method of the servlet. <br>
     * This method is called when a form has its tag value method equals to get.
     *
     * @param request  the request send by the client to the server
     * @param response the response send by the server to the client
     * @throws ServletException if an error occurred
     * @throws IOException      if an error occurred
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        process(request, response);
    }

    /**
     * The doPost method of the servlet. <br>
     * This method is called when a form has its tag value method equals to
     * post.
     *
     * @param request  the request send by the client to the server
     * @param response the response send by the server to the client
     * @throws ServletException if an error occurred
     * @throws IOException      if an error occurred
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        process(request, response);
    }

    /**
     * Initialization of the servlet. <br>
     *
     * @throws ServletException if an error occure
     */
    public void init() throws ServletException {
        // Put your code here
    }

}
